﻿using Microsoft.AspNetCore;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using NLog.Web;
using System;
using WebAPIBase.Service;
using System.Threading.Tasks;

namespace WebAPIBase.API
{
    public class Program
    {
        public static async Task Main(string[] args)
        {
            Console.WriteLine("BaseDirectory:" + AppDomain.CurrentDomain.BaseDirectory);

            Console.WriteLine("current directory:" + System.IO.Directory.GetCurrentDirectory());
            var logger = NLog.Web.NLogBuilder.ConfigureNLog("nlog.config").GetCurrentClassLogger();
            try
            {
                logger.Debug("init main program");               
                //定时任务
                var quartz = new QuartzTask();
                await quartz.RunAsync();

                CreateHostBuilder(args).Build().Run();
            }
            catch (Exception exception)
            {
                //NLog: catch setup errors
                logger.Error(exception, "【Stopped program because of exception】  ");
                throw;
            }
            finally
            {
                // Ensure to flush and stop internal timers/threads before application-exit (Avoid segmentation fault on Linux)
                NLog.LogManager.Shutdown();
            }
        }

        public static IHostBuilder CreateHostBuilder(string[] args) =>
         Host.CreateDefaultBuilder(args)
           .ConfigureWebHostDefaults(webBuilder =>
           {
               webBuilder.UseStartup<Startup>();
           })
           .ConfigureLogging(logging =>
           {
               logging.ClearProviders();
               logging.SetMinimumLevel(Microsoft.Extensions.Logging.LogLevel.Trace);
           })
           .UseNLog();  // NLog: Setup NLog for Dependency injection
    }
}
