﻿using EasyMemoryCache;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.IdentityModel.Tokens;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using Microsoft.OpenApi.Models;
using System;
using WebAPIBase.Data.DapperORM.Class;
using WebAPIBase.Data.DapperORM.Interface;
using WebAPIBase.Model;
using WebAPIBase.Service.Class;
using WebAPIBase.Service.Interface;
using NLog.Extensions.Logging;
using NLog;
using Microsoft.AspNetCore.Diagnostics;
using WebAPIBase.Utils;
using System.Text;
using Sugar.Enties;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Identity.UI;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.FileProviders;
using Quartz;
using Quartz.Impl;
using WebAPIBase.Utils.Jwt;

namespace WebAPIBase.API
{
    public class Startup
    {
        private IWebHostEnvironment _env;
        private const string CONTENTROOTPATHTOKEN = "%CONTENTROOTPATH%";
        public Startup(IConfiguration configuration, IWebHostEnvironment env)
        {
            Configuration = configuration;
            _env = env;
        }

        public IConfiguration Configuration { get; }

        public void ConfigureServices(IServiceCollection services)
        {
            var signingConfigurations = new SigningConfigurations();
            services.AddSingleton(signingConfigurations);
            services.AddSingleton<ISchedulerFactory, StdSchedulerFactory>();

            //var tokenConfigurations = new TokenConfigurations();
            //new ConfigureFromConfigurationOptions<TokenConfigurations>(
            //    Configuration.GetSection("TokenConfigurations"))
            //        .Configure(tokenConfigurations);
            //services.AddSingleton(tokenConfigurations);

            //设置pdf文件存放路径
            if (RequestHelper.IsRemote)
            {
                services.AddSingleton<IFileProvider>(new PhysicalFileProvider(AppSettingsHelper.Configuration["ServiceVirtualPath:RealPath"]));
            }
            else
            {
                services.AddSingleton<IFileProvider>(new PhysicalFileProvider(AppSettingsHelper.Configuration["LocalVirtualPath:RealPath"]));
            }

            // TODO create options classes for app settings
            var identityConn = AppSettingsHelper.Configuration["Logging:AppSettings:ConnectionString"];

            identityConn = identityConn.Replace(CONTENTROOTPATHTOKEN, _env.ContentRootPath);

            services.AddDbContext<IdentityAuthDbContext>(options => options.UseSqlServer(identityConn));
            services.AddDefaultIdentity<AppUser>().AddEntityFrameworkStores<IdentityAuthDbContext>();

            services.AddMemoryCache();

            #region jwt配置
            services.AddTransient<ITokenHelper, TokenHelper>();
            //读取配置文件配置的jwt相关配置
            services.Configure<JWTConfig>(Configuration.GetSection("JWTConfig"));
            //启用JWT
            services.AddAuthentication(Options =>
            {
                Options.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
                Options.DefaultChallengeScheme = JwtBearerDefaults.AuthenticationScheme;
            }).
            AddJwtBearer();
            services.AddScoped<Filter.TokenAuthorize>();
            #endregion


            services.AddLogging(log =>
            {
                log.AddConsole();
                log.AddDebug();
            });
            services.AddAuthorization(auth =>
            {
                auth.AddPolicy("Bearer", new AuthorizationPolicyBuilder()
                    .AddAuthenticationSchemes(JwtBearerDefaults.AuthenticationScheme)
                    .RequireAuthenticatedUser().Build());
            });

            services.AddMvcCore()
                .AddApiExplorer();
            services.AddHttpClient();
            services.AddCors(o => o.AddPolicy("MyPolicy", builder =>
            {
                builder.AllowAnyOrigin()
                    .AllowAnyMethod()
                    .AllowAnyHeader();
            }));



            services.Configure<AppSettings>(Configuration.GetSection("AppSettings"));

            services.AddSingleton<IConfiguration>(Configuration);
            services.AddSingleton<ICaching, Caching>();

            //services.AddAuthorization(options =>
            //{
            //    options.AddPolicy("Member",
            //        policy => policy.RequireClaim("MembershipId"));
            //});

            services.AddSwaggerGen(c =>
            {
                c.SwaggerDoc("v1", new OpenApiInfo { Title = "WebAPIBase API——Netcore 3.0", Version = "v1" });
                
            });
        }

        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            #region pdf存放路径设置
            if (RequestHelper.IsRemote)
            {
                app.UseStaticFiles(new StaticFileOptions()
                {
                    FileProvider = new PhysicalFileProvider(AppSettingsHelper.Configuration["ServiceVirtualPath:RealPath"]),
                    RequestPath = "/WorkFlowPdf"
                });
            }
            else
            {
                app.UseStaticFiles(new StaticFileOptions()
                {
                    FileProvider = new PhysicalFileProvider(AppSettingsHelper.Configuration["LocalVirtualPath:RealPath"]),
                    RequestPath = "/WorkFlowPdf"
                });
            }
            #endregion

            app.UseHttpsRedirection();

            app.UseRouting();
            //启用认证中间件 要写在授权UseAuthorization()的前面
            app.UseAuthentication();
            app.UseAuthorization();

            ///异常捕获的实现
            app.UseCustomErrorMiddleware();

            app.UseCors();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });

            app.UseSwagger();

            app.UseSwaggerUI(c =>
            {
                c.SwaggerEndpoint("/swagger/v1/swagger.json", "ForSignAPIClient v1");
            });
        }
    }
}
